/*
 *
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2021 Timm Fitschen <t.fitschen@indiscale.com>
 * Copyright (C) 2021 IndiScale GmbH <info@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include <gtest/gtest-message.h>   // for Message
#include <gtest/gtest-test-part.h> // for TestPartResult, SuiteApiResolver
#include <memory>                  // for allocator, operator!=, shared_ptr
#include <string>                  // for operator+, string
#include "caosdb/configuration.h"  // for ConfigurationManager
#include "caosdb/connection.h"     // for ConnectionManager, InsecureConnec...
#include "caosdb/exceptions.h"     // for UnknownConnectionError
#include "caosdb_test_utility.h"   // for EXPECT_THROW_MESSAGE, TEST_DATA_DIR
#include "gtest/gtest_pred_impl.h" // for Test, AssertionResult, TestInfo

namespace caosdb::connection {
using caosdb::configuration::ConfigurationManager;

class test_connection : public ::testing::Test {
protected:
  void SetUp() override {
    ConfigurationManager::Clear();
    ConfigurationManager::LoadSingleJSONConfiguration(
      TEST_DATA_DIR + "/test_caosdb_client.json");
  };
  void TearDown() override { ConfigurationManager::Clear(); };
};

TEST_F(test_connection, configure_insecure_localhost_8080) {
  InsecureConnectionConfig config("localhost", 8000);

  EXPECT_EQ("localhost", config.GetHost());
  EXPECT_EQ(8000, config.GetPort());
  auto icc = config.GetChannelCredentials();
  EXPECT_TRUE(icc != nullptr);
}

TEST_F(test_connection, configure_ssl_localhost_8080) {
  auto cacert = PemCertificateProvider("ca chain");
  TlsConnectionConfig config("localhost", 44300, cacert);

  EXPECT_EQ("localhost", config.GetHost());
  EXPECT_EQ(44300, config.GetPort());
  auto sslcc = config.GetChannelCredentials();
  EXPECT_TRUE(sslcc != nullptr);
}

TEST_F(test_connection, connection_manager_unknown_connection) {
  EXPECT_THROW_MESSAGE(ConnectionManager::GetConnection("test"),
                       caosdb::exceptions::UnknownConnectionError,
                       "No connection named 'test' present.");
}

TEST_F(test_connection, connection_manager_get_default_connection) {
  auto connection = ConnectionManager::GetDefaultConnection();
  EXPECT_EQ(connection, ConnectionManager::GetConnection("local-caosdb"));
}

TEST_F(test_connection, connection_manager_get_connection) {

  EXPECT_TRUE(ConnectionManager::GetConnection("local-caosdb-admin"));
}

} // namespace caosdb::connection
